@@ -23,6 +23,18 @@ func TestCmd_template_kubeVersion(t *testing.T) {
 	assert.NotEmpty(t, s)
 }
 
+func TestCmd_template_noApiVersionsInError(t *testing.T) {
+	cmd, err := NewCmdWithVersion(".", HelmV3, false, "")
+	assert.NoError(t, err)
+	_, err = cmd.template("testdata/chart-does-not-exist", &TemplateOpts{
+		KubeVersion: "1.14",
+		APIVersions: []string{"foo", "bar"},
+	})
+	assert.Error(t, err)
+	assert.NotContains(t, err.Error(), "--api-version")
+	assert.ErrorContains(t, err, "<api versions removed> ")
+}
+
 func TestNewCmd_helmV3(t *testing.T) {
 	cmd, err := NewCmd(".", "v3", "")
 	assert.NoError(t, err)
